@@ -7,6 +7,7 @@ __author__ = 'Zitong Lu'
 import numpy as np
 from sklearn.svm import SVC
 from sklearn.model_selection import train_test_split, StratifiedKFold
+from sklearn.preprocessing import StandardScaler
 from neurora.stuff import show_progressbar
 
 np.seterr(divide='ignore', invalid='ignore')
@@ -133,6 +134,8 @@ def tbyt_decoding_kfold(data, labels, n=2, navg=5, time_win=5, time_step=5, nfol
             for j in range(minn):
                 avg_datai[:, j] = np.average(datai[:, j * navg:j * navg + navg], axis=1)
 
+            print(avg_datai[0, 0, :, :5])
+
             x = np.reshape(avg_datai, [n * minn, nchls, newnts])
             y = np.reshape(labelsi, [n * minn])
 
@@ -145,10 +148,11 @@ def tbyt_decoding_kfold(data, labels, n=2, navg=5, time_win=5, time_step=5, nfol
                 fold_index = 0
                 for train_index, test_index in kf.split(xt, y):
 
-                    svm = SVC(kernel='linear')
-                    svm.fit(xt[train_index], y[train_index])
-                    subacc[i, t, fold_index] = svm.score(xt[test_index], y[test_index])
-                    #print(i, t, fold_index, subacc[i, t, fold_index])
+                    scaler = StandardScaler()
+                    x_train = scaler.fit_transform(xt[train_index])
+                    svm = SVC(kernel='linear', tol=1e-4, probability=False)
+                    svm.fit(x_train, y[train_index])
+                    subacc[i, t, fold_index] = svm.score(scaler.transform(xt[test_index]), y[test_index])
 
                     percent = (sub * nrepeats * newnts * nfolds + i * newnts * nfolds + t * nfolds + fold_index) / total * 100
                     show_progressbar("Calculating", percent)
@@ -313,11 +317,14 @@ def tbyt_decoding_holdout(data, labels, n=2, navg=5, time_win=5, time_step=5, it
                 show_progressbar("Calculating", percent)
 
                 state = np.random.randint(0, 100)
+                scaler = StandardScaler()
                 xt = x[:, :, t]
                 x_train, x_test, y_train, y_test = train_test_split(xt, y, test_size=test_size, random_state=state)
-                svm = SVC(kernel='linear')
+                scaler = StandardScaler()
+                x_train = scaler.fit_transform(x_train)
+                svm = SVC(kernel='linear', tol=1e-4, probability=False)
                 svm.fit(x_train, y_train)
-                subacc[i, t] = svm.score(x_test, y_test)
+                subacc[i, t] = svm.score(scaler.transform(x_test), y_test)
 
                 if sub == (nsubs-1) and i == (iter-1) and t == (newnts - 1):
                     print("\nDecoding finished!\n")
@@ -480,9 +487,11 @@ def ct_decoding_kfold(data, labels, n=2, navg=5, time_win=5, time_step=5, nfolds
                 fold_index = 0
                 for train_index, test_index in kf.split(xt, y):
 
-                    svm = SVC(kernel='linear')
-                    svm.fit(xt[train_index], y[train_index])
-                    subacc[i, t, t, fold_index] = svm.score(xt[test_index], y[test_index])
+                    scaler = StandardScaler()
+                    x_train = scaler.fit_transform(xt[train_index])
+                    svm = SVC(kernel='linear', tol=1e-4, probability=False)
+                    svm.fit(x_train, y[train_index])
+                    subacc[i, t, t, fold_index] = svm.score(scaler.transform(xt[test_index]), y[test_index])
 
                     percent = (sub * nrepeats * newnts * nfolds + i * newnts * nfolds + t * nfolds + fold_index) / total * 100
                     show_progressbar("Calculating", percent)
@@ -490,10 +499,10 @@ def ct_decoding_kfold(data, labels, n=2, navg=5, time_win=5, time_step=5, nfolds
                     for tt in range(newnts - 1):
                         if tt < t:
                             xtt = x[:, :, tt]
-                            subacc[i, t, tt, fold_index] = svm.score(xtt[test_index], y[test_index])
+                            subacc[i, t, tt, fold_index] = svm.score(scaler.transform(xtt[test_index]), y[test_index])
                         if tt >= t:
                             xtt = x[:, :, tt + 1]
-                            subacc[i, t, tt+1, fold_index] = svm.score(xtt[test_index], y[test_index])
+                            subacc[i, t, tt+1, fold_index] = svm.score(scaler.transform(xtt[test_index]), y[test_index])
 
                     if sub == (nsubs-1) and i == (nrepeats-1) and t == (newnts - 1) and fold_index == (nfolds - 1):
                         print("\nDecoding finished!\n")
@@ -670,20 +679,22 @@ def ct_decoding_holdout(data, labels, n=2, navg=5, time_win=5, time_step=5, iter
                 state = np.random.randint(0, 100)
                 xt = x[:, :, t]
                 x_train, x_test, y_train, y_test = train_test_split(xt, y, test_size=test_size, random_state=state)
-                svm = SVC(kernel='linear')
+                scaler = StandardScaler()
+                x_train = scaler.fit_transform(x_train)
+                svm = SVC(kernel='linear', tol=1e-4, probability=False)
                 svm.fit(x_train, y_train)
-                subacc[i, t, t] = svm.score(x_test, y_test)
+                subacc[i, t, t] = svm.score(scaler.transform(x_test), y_test)
                 for tt in range(newnts - 1):
                     if tt < t:
                         xtt = x[:, :, tt]
                         x_train, x_testt, y_train, y_test = train_test_split(xtt, y, test_size=test_size,
                                                                              random_state=state)
-                        subacc[i, t, tt] = svm.score(x_testt, y_test)
+                        subacc[i, t, tt] = svm.score(scaler.transform(x_testt), y_test)
                     if tt >= t:
                         xtt = x[:, :, tt + 1]
                         x_train, x_testt, y_train, y_test = train_test_split(xtt, y, test_size=test_size,
                                                                              random_state=state)
-                        subacc[i, t, tt + 1] = svm.score(x_testt, y_test)
+                        subacc[i, t, tt + 1] = svm.score(scaler.transform(x_testt), y_test)
 
                 if sub == (nsubs-1) and i == (iter-1) and t == (newnts - 1):
                     print("\nDecoding finished!\n")
@@ -948,12 +959,14 @@ def unidirectional_transfer_decoding_kfold(data1, labels1, data2, labels2, n=2,
                 fold_index = 0
                 for train_index, test_index in kf.split(xt1, y1):
 
-                    svm = SVC(kernel='linear')
-                    svm.fit(xt1[train_index], y1[train_index])
+                    scaler = StandardScaler()
+                    x_train = scaler.fit_transform(xt1[train_index])
+                    svm = SVC(kernel='linear', tol=1e-4, probability=False)
+                    svm.fit(x_train, y1[train_index])
 
                     for tt in range(newnts2):
                         xt2 = x2[:, :, tt]
-                        subacc[i, t, tt, fold_index] = svm.score(xt2, y2)
+                        subacc[i, t, tt, fold_index] = svm.score(scaler.transform(xt2), y2)
 
                 if sub == (nsubs-1) and i == (nrepeats-1) and t == (newnts1 - 1):
                     print("\nDecoding finished!\n")
@@ -1206,12 +1219,13 @@ def unidirectional_transfer_decoding_holdout(data1, labels1, data2, labels2, n=2
                 percent = (sub * iter * newnts1 + i * newnts1 + t) / total * 100
                 show_progressbar("Calculating", percent)
 
-                xt1 = x1[:, :, t]
-                svm = SVC(kernel='linear')
+                scaler = StandardScaler()
+                xt1 = scaler.fit_transform(x1[:, :, t])
+                svm = SVC(kernel='linear', tol=1e-4, probability=False)
                 svm.fit(xt1, y1)
                 for tt in range(newnts2):
                     xt2 = x2[:, :, tt]
-                    subacc[i, t, tt] = svm.score(xt2, y2)
+                    subacc[i, t, tt] = svm.score(scaler.transform(xt2), y2)
 
                 if sub == (nsubs-1) and i == (iter-1) and t == (newnts1 - 1):
                     print("\nDecoding finished!\n")
@@ -1269,3 +1283,43 @@ def bidirectional_transfer_decoding(data1, labels1, data2, labels2, navg=5, time
         The time-by-time decoding accuracies.
     """
 
+"""data = np.zeros([1, 30, 1, 10])
+
+print(data.shape)
+
+for i in range(30):
+    data[0, i] = i
+
+data[0, 0] = 15
+data[0, 1] = 16
+data[0, 3] = 50
+
+labels = np.zeros([1, 30])
+labels[0, 12:] = 1
+
+acc = tbyt_decoding_kfold(data, labels, n=2, navg=1, time_win=1, time_step=1, nfolds=8, nrepeats=10, smooth=True)
+print(np.average(acc, axis=1))
+
+data = np.zeros([30, 1])
+for i in range(30):
+    data[i] = i
+data[0] = 17
+data[1] = 20
+data[2] = 50
+
+labels = np.zeros([30])
+labels[12:] = 1
+acc = np.zeros([100, 8])
+
+for i in range(100):
+
+    state = np.random.randint(0, 100)
+    kf = StratifiedKFold(n_splits=8, shuffle=True, random_state=state)
+
+    fold_index = 0
+    for train_index, test_index in kf.split(data, labels):
+        svm = SVC(kernel='linear')
+        svm.fit(data[train_index], labels[train_index])
+        acc[i, fold_index] = svm.score(data[test_index], labels[test_index])
+        fold_index= fold_index + 1
+print(np.average(acc, axis=(0, 1)))"""
\ No newline at end of file
