@@ -9,7 +9,8 @@ import nibabel as nib
 from nilearn.image import smooth_img
 import math
 from scipy.stats import t
-from neurora.stuff import fwe_correct, fdr_correct, get_HOcort, get_bg_ch2bet, mask_to
+from neurora.stuff import fwe_correct, fdr_correct, cluster_fwe_correct, cluster_fdr_correct, get_HOcort, get_bg_ch2bet,\
+    mask_to
 from neurora.rsa_plot import plot_brainrsa_rlts
 
 
@@ -280,10 +281,12 @@ def stats_save_nii(corrs, affine, filename=None, corr_mask=get_HOcort(), size=[6
         Only the results those p-values are lower than this value will be visible.
     df : int. Default is 20.
         The degree of freedom.
-    correct_method : None or string 'FWE' or 'FDR'. Default is None.
+    correct_method : None or string 'FWE' or 'FDR' or 'Cluster-FWE' or 'Cluster-FDR'. Default is None.
         The method for correcting the RSA results.
         If correct_method='FWE', here the FWE-correction will be used. If correct_methd='FDR', here the FDR-correction
-        will be used. If correct_method=None, no correction.
+        will be used. If correct_method='Cluster-FWE', here the Cluster-wise FWE-correction will be used. If
+        correct_methd='Cluster-FDR', here the Cluster-wise FDR-correction will be used. If correct_method=None, no
+        correction.
         Only when p<1, correct_method works.
     smooth : bool True or False.  Default is False.
         Smooth the RSA result or not.
@@ -395,6 +398,14 @@ def stats_save_nii(corrs, affine, filename=None, corr_mask=get_HOcort(), size=[6
         if correct_method == "FWE":
             corrsp = fwe_correct(corrsp, p_threshold=p)
 
+        # Cluster-wise FDR-correction
+        if correct_method == "Cluster-FDR":
+            corrsp = cluster_fdr_correct(corrsp, p_threshold=p)
+
+        # Cluster-wise FWE-correction
+        if correct_method == "Cluster-FWE":
+            corrsp = cluster_fwe_correct(corrsp, p_threshold=p)
+
     # iterate through all the calculation units again
 
     # record the valid voxels
