@@ -9,6 +9,7 @@ import numpy as np
 import os
 import math
 from scipy.stats import spearmanr, pearsonr, kendalltau, ttest_1samp, ttest_rel
+from skimage.measure import label
 import sys
 
 # get package abspath
@@ -115,8 +116,8 @@ def fwe_correct(p, p_threshold):
 
     Returns
     -------
-    correctp : array.
-        The FDR corrected p-value map.
+    fwep : array.
+        The FWE corrected p-value map.
     """
 
     px = np.shape(p)[0]
@@ -155,7 +156,7 @@ def fdr_correct(p, p_threshold):
 
     Returns
     -------
-    correctp : array.
+    fdrp : array.
         The FDR corrected p-value map.
     """
 
@@ -210,124 +211,166 @@ def fdr_correct(p, p_threshold):
 
     return fdrp
 
-"""def fdr_correct(p, size=[60, 60, 60], n=64, type="sphere"):
 
-    x = size[0]
-    y = size[1]
-    z = size[2]
+' a function for Cluster-wise FWE-correction for fMRI RSA results '
+
+def cluster_fwe_correct(p, p_threshold):
+
+    """
+    Cluster-wise FWE correction for fMRI RSA results
+
+    Parameters
+    ----------
+    p : array
+        The p-value map (3-D).
+    p_threshold: string
+        The p threshold.
+
+    Returns
+    -------
+    clusterfwep : array.
+        The Cluster-wise FWE corrected p-value map.
+    """
 
     px = np.shape(p)[0]
     py = np.shape(p)[1]
     pz = np.shape(p)[2]
 
-    if type == "cube":
+    n = 0
 
-        n = float(n*px*py*pz/(x*y*z))
+    p01 = np.zeros([px, py, pz])
 
-        nq = 1
-        ni = 1
+    for i in range(px):
+        for j in range(py):
+            for k in range(pz):
 
-        while nq < n:
-            ni = ni + 1
-            nq = ni*ni*ni
+                if (math.isnan(p[i, j, k]) == False):
+                    n = n + 1
+                if p[i, j, k] < p_threshold:
+                    p01[i, j, k] = 1
 
-        n = nq
+    permutation_voxels = np.zeros([10000])
+    for k in range(10000):
+        pi = np.copy(p01)
+        np.random.shuffle(np.reshape(pi, [px * py * pz]))
+        pi = np.reshape(pi, [px, py, pz])
+        labels = label(pi, connectivity=1)
+        nclusters = int(np.max(labels))
+        voxelsinluster = np.zeros([nclusters], dtype=int)
+        for i in range(nclusters):
+            voxelsinluster[i] = str(labels.tolist()).count(str(i + 1))
+        permutation_voxels[k] = max(voxelsinluster)
 
-        print(n, ni)
+    permutation_voxels = np.sort(permutation_voxels)
+    voxels_threshold = permutation_voxels[int(10000*(1-p_threshold))]
 
-        fdrp = np.full([px, py, pz], np.nan)
+    labels = label(p01, connectivity=1)
+    nclusters = int(np.max(labels))
+    voxelsinluster = np.zeros([nclusters], dtype=int)
+    for i in range(nclusters):
+        voxelsinluster[i] = str(labels.tolist()).count(str(i+1))
 
-        for i in range(px-ni+1):
-            for j in range(py-ni+1):
-                for k in range(pz-ni+1):
+    clusterp = np.zeros([nclusters])
+    for i in range(nclusters):
+        clusterp[i] = (10000 - np.max(np.array(np.where(np.sort(np.append(permutation_voxels, voxelsinluster[i])) == voxelsinluster[i])))) / 10000
 
-                    pcluster = p[i:i+ni, j:j+ni, k:k+ni]
-                    pcluster = np.reshape(pcluster, [n])
+    clusterp = clusterp * nclusters
 
-                    index = np.argsort(pcluster)
+    clusterfwep = np.full([px, py, pz], np.nan)
 
-                    for l in range(n):
-                        pcluster[index[l]] = float(pcluster[index[l]]*n/(l+1))
+    for i in range(px):
+        for j in range(py):
+            for k in range(pz):
 
-                    for l in range(n-1):
-                        if pcluster[index[-l-1]] < pcluster[index[-l-2]]:
-                            pcluster[index[-l-2]] = pcluster[index[-l-1]]
+                if (math.isnan(p[i, j, k]) == False) and labels[i, j, k] != 0\
+                        and clusterp[labels[i, j, k]-1] < p_threshold and voxelsinluster[labels[i, j, k]-1] >= voxels_threshold:
+                    clusterfwep[i, j, k] = clusterp[labels[i, j, k]]
 
-                    newpcluster = np.full([n], np.nan)
+    print("finished Cluster-wise FWE correct")
 
-                    for l in range(n):
-                        newpcluster[l] = pcluster[index[l]]
+    return clusterfwep
 
-                    fdrp[i:i+ni, j:j+ni, k:k+ni] = np.reshape(newpcluster, [ni, ni, ni])
 
-        print("finished FDR correct")
+' a function for Cluster-wise FDR-correction for fMRI RSA results '
 
-        return fdrp
+def cluster_fdr_correct(p, p_threshold):
 
-    elif type == "sphere":
+    """
+    Cluster-wise FDR correction for fMRI RSA results
 
-        ni = int(nr * px * py * pz / (x * y * z)) + 1
+    Parameters
+    ----------
+    p : array
+        The p-value map (3-D).
+    p_threshold: string
+        The p threshold.
 
-        c0 = [int(ni), int(ni), int(ni)]
-        n0 = 0
-        for i in range(int(2 * ni + 1)):
-            for j in range(int(2 * ni + 1)):
-                for k in range(int(2 * ni + 1)):
-                    dist = np.square(i - c0[0]) + np.square(j - c0[1]) + np.square(k - c0[2])
-                    if dist <= ni * ni:
-                        # print(i, j)
-                        n0 = n0 + 1
+    Returns
+    -------
+    clusterfdrp : array.
+        The Cluster-wise FDR corrected p-value map.
+    """
 
-        n = n0
-        # print(c0)
-        # print(n0)
-        print(n, ni)
+    px = np.shape(p)[0]
+    py = np.shape(p)[1]
+    pz = np.shape(p)[2]
 
-        fdrp = np.full([px, py, pz], np.nan)
+    n = 0
 
-        for i in range(px - 2 * ni):
-            for j in range(py - 2 * ni):
-                for k in range(pz - 2 * ni):
+    p01 = np.zeros([px, py, pz])
 
-                    nindex = 0
+    for i in range(px):
+        for j in range(py):
+            for k in range(pz):
 
-                    pcluster = np.zeros([n], dtype=np.float)
+                if (math.isnan(p[i, j, k]) == False):
+                    n = n + 1
+                if p[i, j, k] < p_threshold:
+                    p01[i, j, k] = 1
 
-                    for lx in range(2 * ni + 1):
-                        for ly in range(2 * ni + 1):
-                            for lz in range(2 * ni + 1):
-                                dist = np.square(lx - c0[0]) + np.square(ly - c0[1]) + np.square(lz - c0[2])
-                                if dist <= ni * ni:
-                                    pcluster[nindex] = p[i + lx, j + ly, k + lz]
-                                    nindex = nindex + 1
+    permutation_voxels = np.zeros([10000])
+    for k in range(10000):
+        pi = np.copy(p01)
+        np.random.shuffle(np.reshape(pi, [px * py * pz]))
+        pi = np.reshape(pi, [px, py, pz])
+        labels = label(pi, connectivity=1)
+        nclusters = int(np.max(labels))
+        voxelsinluster = np.zeros([nclusters], dtype=int)
+        for i in range(nclusters):
+            voxelsinluster[i] = str(labels.tolist()).count(str(i + 1))
+        permutation_voxels[k] = max(voxelsinluster)
 
-                    index = np.argsort(pcluster)
+    permutation_voxels = np.sort(permutation_voxels)
+    voxels_threshold = permutation_voxels[int(10000*(1-p_threshold))]
 
-                    for l in range(n):
-                        pcluster[index[l]] = float(pcluster[index[l]] * n / (l + 1))
+    labels = label(p01, connectivity=1)
+    nclusters = int(np.max(labels))
+    voxelsinluster = np.zeros([nclusters], dtype=int)
+    for i in range(nclusters):
+        voxelsinluster[i] = str(labels.tolist()).count(str(i+1))
 
-                    for l in range(n - 1):
-                        if pcluster[index[-l - 1]] < pcluster[index[-l - 2]]:
-                            pcluster[index[-l - 2]] = pcluster[index[-l - 1]]
+    clusterp = np.zeros([nclusters])
+    for i in range(nclusters):
+        clusterp[i] = (10000 - np.max(np.array(np.where(np.sort(np.append(permutation_voxels, voxelsinluster[i])) == voxelsinluster[i])))) / 10000
 
-                    newpcluster = np.full([n], np.nan)
+    index = np.argsort(clusterp)
 
-                    for l in range(n):
-                        newpcluster[l] = pcluster[index[l]]
+    for i in range(nclusters):
+        clusterp[index[i]] = float(clusterp[index[i]] * nclusters / (i+1))
 
-                    nindex = 0
+    clusterfdrp = np.full([px, py, pz], np.nan)
 
-                    for lx in range(2 * ni + 1):
-                        for ly in range(2 * ni + 1):
-                            for lz in range(2 * ni + 1):
-                                dist = np.square(lx - c0[0]) + np.square(ly - c0[1]) + np.square(lz - c0[2])
-                                if dist <= ni * ni:
-                                    fdrp[i + lx, j + ly, k + lz] = newpcluster[nindex]
-                                    nindex = nindex + 1
+    for i in range(px):
+        for j in range(py):
+            for k in range(pz):
 
-        print("finished FDR correct")
+                if (math.isnan(p[i, j, k]) == False) and labels[i, j, k] != 0\
+                        and clusterp[labels[i, j, k]-1] < p_threshold and voxelsinluster[labels[i, j, k]-1] >= voxels_threshold:
+                    clusterfdrp[i, j, k] = clusterp[labels[i, j, k]]
 
-        return fdrp"""
+    print("finished Cluster-wise FDR correct")
+
+    return clusterfdrp
 
 
 ' a function for fMRI RSA results correction by threshold '
